# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-05-02 04:21
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ExampleItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enabled', models.BooleanField()),
                ('name', models.CharField(max_length=30)),
            ],
            options={
                'ordering': ['pk'],
                'permissions': (('view_exampleitem', 'Can view example item'),),
            },
        ),
        migrations.CreateModel(
            name='ExampleM2MItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.IntegerField(default=1)),
                ('items', models.ManyToManyField(related_name='items', to='rest_framework_reactive_tests.ExampleItem')),
            ],
            options={
                'ordering': ['pk'],
            },
        ),
        migrations.CreateModel(
            name='ExampleSubItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enabled', models.BooleanField()),
                ('parent', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='rest_framework_reactive_tests.ExampleItem')),
            ],
            options={
                'ordering': ['pk'],
                'permissions': (('view_examplesubitem', 'Can view example sub item'),),
            },
        ),
    ]
